@@ -61,7 +61,11 @@ class Load(Generative, MapperOption):
 
     .. seealso::
 
-        :ref:`loading_toplevel`
+        :ref:`deferred_options`
+
+        :ref:`deferred_loading_w_multiple`
+
+        :ref:`relationship_loader_options`
 
     """
 
@@ -180,10 +184,11 @@ class Load(Generative, MapperOption):
         else:
             query._attributes.update(self.context)
 
-    def _generate_path(self, path, attr, wildcard_key, raiseerr=True):
+    def _generate_path(
+        self, path, attr, for_strategy, wildcard_key, raiseerr=True
+    ):
         existing_of_type = self._of_type
         self._of_type = None
-
         if raiseerr and not path.has_entity:
             if isinstance(path, TokenRegistry):
                 raise sa_exc.ArgumentError(
@@ -191,9 +196,8 @@ class Load(Generative, MapperOption):
                 )
             else:
                 raise sa_exc.ArgumentError(
-                    "Attribute '%s' of entity '%s' does not "
-                    "refer to a mapped entity"
-                    % (path.prop.key, path.parent.entity)
+                    "Mapped attribute '%s' does not "
+                    "refer to a mapped entity" % (path.prop,)
                 )
 
         if isinstance(attr, util.string_types):
@@ -203,6 +207,13 @@ class Load(Generative, MapperOption):
                     self.propagate_to_loaders = False
                 if wildcard_key:
                     attr = "%s:%s" % (wildcard_key, attr)
+
+                # TODO: AliasedInsp inside the path for of_type is not
+                # working for a with_polymorphic entity because the
+                # relationship loaders don't render the with_poly into the
+                # path.  See #4469 which will try to improve this
+                if existing_of_type and not existing_of_type.is_aliased_class:
+                    path = path.parent[existing_of_type]
                 path = path.token(attr)
                 self.path = path
                 return path
@@ -211,6 +222,7 @@ class Load(Generative, MapperOption):
                 ent = inspect(existing_of_type)
             else:
                 ent = path.entity
+
             try:
                 # use getattr on the class to work around
                 # synonyms, hybrids, etc.
@@ -218,17 +230,22 @@ class Load(Generative, MapperOption):
             except AttributeError:
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Can't find property named '%s' on the "
-                        "mapped entity %s in this Query. " % (attr, ent)
+                        'Can\'t find property named "%s" on '
+                        "%s in this Query." % (attr, ent)
                     )
                 else:
                     return None
             else:
-                attr = attr.property
+                attr = found_property = attr.property
 
             path = path[attr]
         elif _is_mapped_class(attr):
-            if not attr.common_parent(path.mapper):
+            # TODO: this does not appear to be a valid codepath.  "attr"
+            # would never be a mapper.  This block is present in 1.2
+            # as well however does not seem to be accessed in any tests.
+            if not orm_util._entity_corresponds_to_use_path_impl(
+                attr.parent, path[-1]
+            ):
                 if raiseerr:
                     raise sa_exc.ArgumentError(
                         "Attribute '%s' does not "
@@ -237,13 +254,28 @@ class Load(Generative, MapperOption):
                 else:
                     return None
         else:
-            prop = attr.property
+            prop = found_property = attr.property
 
-            if not prop.parent.common_parent(path.mapper):
+            if not orm_util._entity_corresponds_to_use_path_impl(
+                attr.parent, path[-1]
+            ):
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Attribute '%s' does not "
-                        "link from element '%s'" % (attr, path.entity)
+                        'Attribute "%s" does not '
+                        'link from element "%s".%s'
+                        % (
+                            attr,
+                            path.entity,
+                            (
+                                "  Did you mean to use "
+                                "%s.of_type(%s)?"
+                                % (path[-2], attr.class_.__name__)
+                                if len(path) > 1
+                                and path.entity.is_mapper
+                                and attr.parent.is_aliased_class
+                                else ""
+                            ),
+                        )
                     )
                 else:
                     return None
@@ -264,39 +296,20 @@ class Load(Generative, MapperOption):
                         _existing_alias=existing,
                     )
                     ext_info = inspect(ac)
-                elif not ext_info.with_polymorphic_mappers:
-                    ext_info = orm_util.AliasedInsp(
-                        ext_info.entity,
-                        ext_info.mapper.base_mapper,
-                        ext_info.selectable,
-                        ext_info.name,
-                        ext_info.with_polymorphic_mappers or [ext_info.mapper],
-                        ext_info.polymorphic_on,
-                        ext_info._base_alias,
-                        ext_info._use_mapper_path,
-                        ext_info._adapt_on_names,
-                        ext_info.represents_outer_join,
-                    )
 
                 path.entity_path[prop].set(
                     self.context, "path_with_polymorphic", ext_info
                 )
 
-                # the path here will go into the context dictionary and
-                # needs to match up to how the class graph is traversed.
-                # so we can't put an AliasedInsp in the path here, needs
-                # to be the base mapper.
-                path = path[prop][ext_info.mapper]
+                path = path[prop][ext_info]
 
-                # but, we need to know what the original of_type()
-                # argument is for cache key purposes.  so....store that too.
-                # it might be better for "path" to really represent,
-                # "the path", but trying to keep the impact of the cache
-                # key feature localized for now
                 self._of_type = of_type_info
+
             else:
                 path = path[prop]
 
+        if for_strategy is not None:
+            found_property._get_strategy(for_strategy)
         if path.has_entity:
             path = path.entity_path
         self.path = path
@@ -310,16 +323,63 @@ class Load(Generative, MapperOption):
             strategy = tuple(sorted(strategy.items()))
         return strategy
 
+    def _apply_to_parent(self, parent, applied, bound):
+        raise NotImplementedError(
+            "Only 'unbound' loader options may be used with the "
+            "Load.options() method"
+        )
+
+    @_generative
+    def options(self, *opts):
+        r"""Apply a series of options as sub-options to this :class:`.Load`
+        object.
+
+        E.g.::
+
+            query = session.query(Author)
+            query = query.options(
+                        joinedload(Author.book).options(
+                            load_only("summary", "excerpt"),
+                            joinedload(Book.citations).options(
+                                joinedload(Citation.author)
+                            )
+                        )
+                    )
+
+        :param \*opts: A series of loader option objects (ultimately
+         :class:`.Load` objects) which should be applied to the path
+         specified by this :class:`.Load` object.
+
+        .. versionadded:: 1.3.6
+
+        .. seealso::
+
+            :func:`.defaultload`
+
+            :ref:`relationship_loader_options`
+
+            :ref:`deferred_loading_w_multiple`
+
+        """
+        apply_cache = {}
+        bound = not isinstance(self, _UnboundLoad)
+        if bound:
+            raise NotImplementedError(
+                "The options() method is currently only supported "
+                "for 'unbound' loader options"
+            )
+        for opt in opts:
+            opt._apply_to_parent(self, apply_cache, bound)
+
     @_generative
     def set_relationship_strategy(
         self, attr, strategy, propagate_to_loaders=True
     ):
         strategy = self._coerce_strat(strategy)
-
         self.is_class_strategy = False
         self.propagate_to_loaders = propagate_to_loaders
         # if the path is a wildcard, this will set propagate_to_loaders=False
-        self._generate_path(self.path, attr, "relationship")
+        self._generate_path(self.path, attr, strategy, "relationship")
         self.strategy = strategy
         if strategy is not None:
             self._set_path_strategy()
@@ -332,7 +392,7 @@ class Load(Generative, MapperOption):
         for attr in attrs:
             cloned = self._generate()
             cloned.strategy = strategy
-            cloned._generate_path(self.path, attr, "column")
+            cloned._generate_path(self.path, attr, strategy, "column")
             cloned.propagate_to_loaders = True
             if opts:
                 cloned.local_opts.update(opts)
@@ -346,7 +406,7 @@ class Load(Generative, MapperOption):
         strategy = self._coerce_strat(strategy)
 
         for attr in attrs:
-            path = self._generate_path(self.path, attr, None)
+            path = self._generate_path(self.path, attr, strategy, None)
             cloned = self._generate()
             cloned.strategy = strategy
             cloned.path = path
@@ -358,7 +418,7 @@ class Load(Generative, MapperOption):
         strategy = self._coerce_strat(strategy)
         cloned = self._generate()
         cloned.is_class_strategy = True
-        path = cloned._generate_path(self.path, None, None)
+        path = cloned._generate_path(self.path, None, strategy, None)
         cloned.strategy = strategy
         cloned.path = path
         cloned.propagate_to_loaders = True
@@ -386,12 +446,21 @@ class Load(Generative, MapperOption):
         else:
             effective_path = self.path
 
-        self._set_for_path(
-            self.context,
-            effective_path,
-            replace=True,
-            merge_opts=self.is_opts_only,
-        )
+        if effective_path.is_token:
+            for path in effective_path.generate_for_superclasses():
+                self._set_for_path(
+                    self.context,
+                    path,
+                    replace=True,
+                    merge_opts=self.is_opts_only,
+                )
+        else:
+            self._set_for_path(
+                self.context,
+                effective_path,
+                replace=True,
+                merge_opts=self.is_opts_only,
+            )
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -436,7 +505,7 @@ class _UnboundLoad(Load):
     """Represent a loader option that isn't tied to a root entity.
 
     The loader option will produce an entity-linked :class:`.Load`
-    object when it is passed :metfh:`.Query.options`.
+    object when it is passed :meth:`.Query.options`.
 
     This provides compatibility with the traditional system
     of freestanding options, e.g. ``joinedload('x.y.z')``.
@@ -472,7 +541,42 @@ class _UnboundLoad(Load):
     def _set_path_strategy(self):
         self._to_bind.append(self)
 
-    def _generate_path(self, path, attr, wildcard_key):
+    def _apply_to_parent(self, parent, applied, bound):
+        if self in applied:
+            return applied[self]
+
+        cloned = self._generate()
+
+        applied[self] = cloned
+
+        cloned.strategy = self.strategy
+        if self.path:
+            attr = self.path[-1]
+            if isinstance(attr, util.string_types) and attr.endswith(
+                _DEFAULT_TOKEN
+            ):
+                attr = attr.split(":")[0] + ":" + _WILDCARD_TOKEN
+            cloned._generate_path(
+                parent.path + self.path[0:-1], attr, self.strategy, None
+            )
+
+        # these assertions can go away once the "sub options" API is
+        # mature
+        assert cloned.propagate_to_loaders == self.propagate_to_loaders
+        assert cloned.is_class_strategy == self.is_class_strategy
+        assert cloned.is_opts_only == self.is_opts_only
+
+        new_to_bind = {
+            elem._apply_to_parent(parent, applied, bound)
+            for elem in self._to_bind
+        }
+        cloned._to_bind = parent._to_bind
+        cloned._to_bind.extend(new_to_bind)
+        cloned.local_opts.update(self.local_opts)
+
+        return cloned
+
+    def _generate_path(self, path, attr, for_strategy, wildcard_key):
         if (
             wildcard_key
             and isinstance(attr, util.string_types)
@@ -660,7 +764,7 @@ class _UnboundLoad(Load):
         elif isinstance(token, PropComparator):
             prop = token.property
             entity = self._find_entity_prop_comparator(
-                entities, prop.key, token._parententity, raiseerr
+                entities, prop, token._parententity, raiseerr
             )
         elif self.is_class_strategy and _is_mapped_class(token):
             entity = inspect(token)
@@ -693,9 +797,13 @@ class _UnboundLoad(Load):
         path = loader.path
 
         if not loader.is_class_strategy:
-            for token in start_path:
+            for idx, token in enumerate(start_path):
                 if not loader._generate_path(
-                    loader.path, token, None, raiseerr
+                    loader.path,
+                    token,
+                    self.strategy if idx == len(start_path) - 1 else None,
+                    None,
+                    raiseerr,
                 ):
                     return
 
@@ -728,7 +836,7 @@ class _UnboundLoad(Load):
 
         return loader
 
-    def _find_entity_prop_comparator(self, entities, token, mapper, raiseerr):
+    def _find_entity_prop_comparator(self, entities, prop, mapper, raiseerr):
         if _is_aliased_class(mapper):
             searchfor = mapper
         else:
@@ -740,15 +848,18 @@ class _UnboundLoad(Load):
             if raiseerr:
                 if not list(entities):
                     raise sa_exc.ArgumentError(
-                        "Query has only expression-based entities - "
-                        "can't find property named '%s'." % (token,)
+                        "Query has only expression-based entities, "
+                        'which do not apply to %s "%s"'
+                        % (util.clsname_as_plain_name(type(prop)), prop)
                     )
                 else:
                     raise sa_exc.ArgumentError(
-                        "Can't find property '%s' on any entity "
-                        "specified in this Query.  Note the full path "
-                        "from root (%s) to target entity must be specified."
-                        % (token, ",".join(str(x) for x in entities))
+                        'Mapped attribute "%s" does not apply to any of the '
+                        "root entities in this query, e.g. %s. Please "
+                        "specify the full path "
+                        "from one of the root entities to the target "
+                        "attribute. "
+                        % (prop, ", ".join(str(x) for x in entities))
                     )
             else:
                 return None
@@ -758,9 +869,17 @@ class _UnboundLoad(Load):
             if len(list(entities)) != 1:
                 if raiseerr:
                     raise sa_exc.ArgumentError(
-                        "Wildcard loader can only be used with exactly "
-                        "one entity.  Use Load(ent) to specify "
-                        "specific entities."
+                        "Can't apply wildcard ('*') or load_only() "
+                        "loader option to multiple entities %s. Specify "
+                        "loader options for each entity individually, such "
+                        "as %s."
+                        % (
+                            ", ".join(str(ent) for ent in entities),
+                            ", ".join(
+                                "Load(%s).some_option('*')" % ent
+                                for ent in entities
+                            ),
+                        )
                     )
         elif token.endswith(_DEFAULT_TOKEN):
             raiseerr = False
@@ -774,7 +893,7 @@ class _UnboundLoad(Load):
             if raiseerr:
                 raise sa_exc.ArgumentError(
                     "Query has only expression-based entities - "
-                    "can't find property named '%s'." % (token,)
+                    'can\'t find property named "%s".' % (token,)
                 )
             else:
                 return None
@@ -809,7 +928,6 @@ See :func:`.orm.%(name)s` for usage examples.
         return self
 
     def _add_unbound_all_fn(self, fn):
-        self._unbound_all_fn = fn
         fn.__doc__ = """Produce a standalone "all" option for :func:`.orm.%(name)s`.
 
 .. deprecated:: 0.9
@@ -825,6 +943,15 @@ See :func:`.orm.%(name)s` for usage examples.
 """ % {
             "name": self.name
         }
+        fn = util.deprecated(
+            "0.9",
+            "The :func:`.%(name)s_all` function is deprecated, and will be "
+            "removed in a future release.  Please use method chaining with "
+            ":func:`.%(name)s` instead" % {"name": self.name},
+            add_deprecation_to_docstring=False,
+        )(fn)
+
+        self._unbound_all_fn = fn
         return self
 
 
@@ -1283,6 +1410,10 @@ def defaultload(loadopt, attr):
 
     .. seealso::
 
+        :meth:`.Load.options` - allows for complex hierarchical
+        loader option structures with less verbosity than with individual
+        :func:`.defaultload` directives.
+
         :ref:`relationship_loader_options`
 
         :ref:`deferred_loading_w_multiple`
@@ -1298,8 +1429,8 @@ def defaultload(*keys):
 
 @loader_option()
 def defer(loadopt, key):
-    r"""Indicate that the given column-oriented attribute should be deferred, e.g.
-    not loaded until accessed.
+    r"""Indicate that the given column-oriented attribute should be deferred,
+    e.g. not loaded until accessed.
 
     This function is part of the :class:`.Load` interface and supports
     both method-chained and standalone operation.
@@ -1337,10 +1468,16 @@ def defer(loadopt, key):
 
     :param key: Attribute to be deferred.
 
-    :param \*addl_attrs: Deprecated; this option supports the old 0.8 style
+    :param \*addl_attrs: This option supports the old 0.8 style
      of specifying a path as a series of attributes, which is now superseded
      by the method-chained style.
 
+        .. deprecated:: 0.9  The \*addl_attrs on :func:`.orm.defer` is
+           deprecated and will be removed in a future release.   Please
+           use method chaining in conjunction with defaultload() to
+           indicate a path.
+
+
     .. seealso::
 
         :ref:`deferred`
@@ -1355,6 +1492,12 @@ def defer(loadopt, key):
 
 @defer._add_unbound_fn
 def defer(key, *addl_attrs):
+    if addl_attrs:
+        util.warn_deprecated(
+            "The *addl_attrs on orm.defer is deprecated.  Please use "
+            "method chaining in conjunction with defaultload() to "
+            "indicate a path."
+        )
     return _UnboundLoad._from_keys(
         _UnboundLoad.defer, (key,) + addl_attrs, False, {}
     )
@@ -1380,12 +1523,21 @@ def undefer(loadopt, key):
         session.query(MyClass, MyOtherClass).options(
             Load(MyClass).undefer("*"))
 
+        # undefer a column on a related object
+        session.query(MyClass).options(
+            defaultload(MyClass.items).undefer('text'))
+
     :param key: Attribute to be undeferred.
 
-    :param \*addl_attrs: Deprecated; this option supports the old 0.8 style
+    :param \*addl_attrs: This option supports the old 0.8 style
      of specifying a path as a series of attributes, which is now superseded
      by the method-chained style.
 
+        .. deprecated:: 0.9  The \*addl_attrs on :func:`.orm.undefer` is
+           deprecated and will be removed in a future release.   Please
+           use method chaining in conjunction with defaultload() to
+           indicate a path.
+
     .. seealso::
 
         :ref:`deferred`
@@ -1402,6 +1554,12 @@ def undefer(loadopt, key):
 
 @undefer._add_unbound_fn
 def undefer(key, *addl_attrs):
+    if addl_attrs:
+        util.warn_deprecated(
+            "The *addl_attrs on orm.undefer is deprecated.  Please use "
+            "method chaining in conjunction with defaultload() to "
+            "indicate a path."
+        )
     return _UnboundLoad._from_keys(
         _UnboundLoad.undefer, (key,) + addl_attrs, False, {}
     )
