@@ -42,8 +42,10 @@ class ReflectionTest(fixtures.TestBase, ComparesTables, AssertsCompiledSQL):
             Column("user_name", types.VARCHAR(20), nullable=False),
             Column("test1", types.CHAR(5), nullable=False),
             Column("test2", types.Float(5), nullable=False),
+            Column("test2.5", types.Float(), nullable=False),
             Column("test3", types.Text()),
             Column("test4", types.Numeric, nullable=False),
+            Column("test4.5", types.Numeric(10, 2), nullable=False),
             Column("test5", types.DateTime),
             Column(
                 "parent_user_id",
@@ -105,6 +107,19 @@ class ReflectionTest(fixtures.TestBase, ComparesTables, AssertsCompiledSQL):
     def test_money_type(self):
         self._test_specific_type(mssql.MONEY, "MONEY")
 
+    def test_numeric_prec_scale(self):
+        self._test_specific_type(mssql.NUMERIC(10, 2), "NUMERIC(10, 2)")
+
+    def test_float(self):
+        self._test_specific_type(mssql.FLOAT, "FLOAT(53)")
+
+    def test_real(self):
+        self._test_specific_type(mssql.REAL, "REAL")
+
+    def test_float_as_real(self):
+        # FLOAT(5) comes back as REAL
+        self._test_specific_type(mssql.FLOAT(5), "REAL")
+
     @testing.provide_metadata
     def test_identity(self):
         metadata = self.metadata
@@ -193,6 +208,23 @@ class ReflectionTest(fixtures.TestBase, ComparesTables, AssertsCompiledSQL):
             ],
         )
 
+    @testing.provide_metadata
+    def test_table_name_that_is_greater_than_16_chars(self):
+        metadata = self.metadata
+        Table(
+            "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("foo", Integer),
+            Index("foo_idx", "foo"),
+        )
+        metadata.create_all()
+
+        t = Table(
+            "ABCDEFGHIJKLMNOPQRSTUVWXYZ", MetaData(), autoload_with=testing.db
+        )
+        eq_(t.name, "ABCDEFGHIJKLMNOPQRSTUVWXYZ")
+
     @testing.provide_metadata
     def test_db_qualified_items(self):
         metadata = self.metadata
