@@ -6,7 +6,9 @@ import os
 
 from sqlalchemy import and_
 from sqlalchemy import bindparam
+from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import column
 from sqlalchemy import create_engine
 from sqlalchemy import DefaultClause
 from sqlalchemy import event
@@ -25,6 +27,7 @@ from sqlalchemy import sql
 from sqlalchemy import Table
 from sqlalchemy import testing
 from sqlalchemy import text
+from sqlalchemy import tuple_
 from sqlalchemy import types as sqltypes
 from sqlalchemy import UniqueConstraint
 from sqlalchemy import util
@@ -115,7 +118,7 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
                 ValueError,
                 "Couldn't parse %s string." % disp,
                 lambda: testing.db.execute(
-                    text("select 'ASDF' as value", typemap={"value": typ})
+                    text("select 'ASDF' as value").columns(value=typ)
                 ).scalar(),
             )
 
@@ -253,12 +256,12 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
 
         dialect = sqlite.dialect()
         for t in (
-            String(convert_unicode=True),
-            sqltypes.CHAR(convert_unicode=True),
+            String(),
+            sqltypes.CHAR(),
             sqltypes.Unicode(),
             sqltypes.UnicodeText(),
-            String(convert_unicode=True),
-            sqltypes.CHAR(convert_unicode=True),
+            String(),
+            sqltypes.CHAR(),
             sqltypes.Unicode(),
             sqltypes.UnicodeText(),
         ):
@@ -268,6 +271,54 @@ class TestTypes(fixtures.TestBase, AssertsExecutionResults):
             )
 
 
+class JSONTest(fixtures.TestBase):
+
+    __requires__ = ("json_type",)
+    __only_on__ = "sqlite"
+
+    @testing.provide_metadata
+    @testing.requires.reflects_json_type
+    def test_reflection(self):
+        Table("json_test", self.metadata, Column("foo", sqlite.JSON))
+        self.metadata.create_all()
+
+        reflected = Table("json_test", MetaData(), autoload_with=testing.db)
+        is_(reflected.c.foo.type._type_affinity, sqltypes.JSON)
+        assert isinstance(reflected.c.foo.type, sqlite.JSON)
+
+    @testing.provide_metadata
+    def test_rudimentary_roundtrip(self):
+        sqlite_json = Table(
+            "json_test", self.metadata, Column("foo", sqlite.JSON)
+        )
+
+        self.metadata.create_all()
+
+        value = {"json": {"foo": "bar"}, "recs": ["one", "two"]}
+
+        with testing.db.connect() as conn:
+            conn.execute(sqlite_json.insert(), foo=value)
+
+            eq_(conn.scalar(select([sqlite_json.c.foo])), value)
+
+    @testing.provide_metadata
+    def test_extract_subobject(self):
+        sqlite_json = Table(
+            "json_test", self.metadata, Column("foo", sqlite.JSON)
+        )
+
+        self.metadata.create_all()
+
+        value = {"json": {"foo": "bar"}}
+
+        with testing.db.connect() as conn:
+            conn.execute(sqlite_json.insert(), foo=value)
+
+            eq_(
+                conn.scalar(select([sqlite_json.c.foo["json"]])), value["json"]
+            )
+
+
 class DateTimeTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_time_microseconds(self):
         dt = datetime.datetime(2008, 6, 27, 12, 0, 0, 125)
@@ -455,12 +506,49 @@ class DefaultsTest(fixtures.TestBase, AssertsCompiledSQL):
             Column("x", Boolean, server_default=sql.false()),
         )
         t.create(testing.db)
-        testing.db.execute(t.insert())
-        testing.db.execute(t.insert().values(x=True))
-        eq_(
-            testing.db.execute(t.select().order_by(t.c.x)).fetchall(),
-            [(False,), (True,)],
+        with testing.db.connect() as conn:
+            conn.execute(t.insert())
+            conn.execute(t.insert().values(x=True))
+            eq_(
+                conn.execute(t.select().order_by(t.c.x)).fetchall(),
+                [(False,), (True,)],
+            )
+
+    @testing.provide_metadata
+    def test_function_default(self):
+        t = Table(
+            "t",
+            self.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("x", DateTime(), server_default=func.now()),
         )
+        t.create(testing.db)
+        with testing.db.connect() as conn:
+            now = conn.scalar(func.now())
+            today = datetime.datetime.today()
+            conn.execute(t.insert())
+            conn.execute(t.insert().values(x=today))
+            eq_(
+                conn.execute(select([t.c.x]).order_by(t.c.id)).fetchall(),
+                [(now,), (today,)],
+            )
+
+    @testing.provide_metadata
+    def test_expression_with_function_default(self):
+        t = Table(
+            "t",
+            self.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("x", Integer(), server_default=func.abs(-5) + 17),
+        )
+        t.create(testing.db)
+        with testing.db.connect() as conn:
+            conn.execute(t.insert())
+            conn.execute(t.insert().values(x=35))
+            eq_(
+                conn.execute(select([t.c.x]).order_by(t.c.id)).fetchall(),
+                [(22,), (35,)],
+            )
 
     def test_old_style_default(self):
         """test non-quoted integer value on older sqlite pragma"""
@@ -802,6 +890,36 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             ")",
         )
 
+    def test_column_defaults_ddl(self):
+
+        t = Table(
+            "t", MetaData(), Column("x", Boolean, server_default=sql.false())
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE t (x BOOLEAN DEFAULT (0), CHECK (x IN (0, 1)))",
+        )
+
+        t = Table(
+            "t",
+            MetaData(),
+            Column("x", String(), server_default=func.sqlite_version()),
+        )
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE t (x VARCHAR DEFAULT (sqlite_version()))",
+        )
+
+        t = Table(
+            "t",
+            MetaData(),
+            Column("x", Integer(), server_default=func.abs(-5) + 17),
+        )
+        self.assert_compile(
+            CreateTable(t), "CREATE TABLE t (x INTEGER DEFAULT (abs(-5) + 17))"
+        )
+
     def test_create_partial_index(self):
         m = MetaData()
         tbl = Table("testtbl", m, Column("data", Integer))
@@ -846,6 +964,187 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect=sqlite.dialect(),
         )
 
+    def test_in_tuple(self):
+        self.assert_compile(
+            tuple_(column("q"), column("p")).in_([(1, 2), (3, 4)]),
+            "(q, p) IN (VALUES (?, ?), (?, ?))",
+        )
+
+
+class OnConflictDDLTest(fixtures.TestBase, AssertsCompiledSQL):
+
+    __dialect__ = sqlite.dialect()
+
+    def test_on_conflict_clause_column_not_null(self):
+        c = Column(
+            "test", Integer, nullable=False, sqlite_on_conflict_not_null="FAIL"
+        )
+
+        self.assert_compile(
+            schema.CreateColumn(c),
+            "test INTEGER NOT NULL " "ON CONFLICT FAIL",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_column_many_clause(self):
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column(
+                "test",
+                Integer,
+                nullable=False,
+                primary_key=True,
+                sqlite_on_conflict_not_null="FAIL",
+                sqlite_on_conflict_primary_key="IGNORE",
+            ),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n ("
+            "test INTEGER NOT NULL ON CONFLICT FAIL, "
+            "PRIMARY KEY (test) ON CONFLICT IGNORE)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_unique_constraint_from_column(self):
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column(
+                "x", String(30), unique=True, sqlite_on_conflict_unique="FAIL"
+            ),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n (x VARCHAR(30), " "UNIQUE (x) ON CONFLICT FAIL)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_unique_constraint(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column("id", Integer),
+            Column("x", String(30)),
+            UniqueConstraint("id", "x", sqlite_on_conflict="FAIL"),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n (id INTEGER, x VARCHAR(30), "
+            "UNIQUE (id, x) ON CONFLICT FAIL)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_primary_key(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column(
+                "id",
+                Integer,
+                primary_key=True,
+                sqlite_on_conflict_primary_key="FAIL",
+            ),
+            sqlite_autoincrement=True,
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n (id INTEGER NOT NULL "
+            "PRIMARY KEY ON CONFLICT FAIL AUTOINCREMENT)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_primary_key_constraint_from_column(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column(
+                "x",
+                String(30),
+                sqlite_on_conflict_primary_key="FAIL",
+                primary_key=True,
+            ),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n (x VARCHAR(30) NOT NULL, "
+            "PRIMARY KEY (x) ON CONFLICT FAIL)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_check_constraint(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column("id", Integer),
+            Column("x", Integer),
+            CheckConstraint("id > x", sqlite_on_conflict="FAIL"),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n (id INTEGER, x INTEGER, "
+            "CHECK (id > x) ON CONFLICT FAIL)",
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_check_constraint_from_column(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column(
+                "x",
+                Integer,
+                CheckConstraint("x > 1", sqlite_on_conflict="FAIL"),
+            ),
+        )
+
+        assert_raises_message(
+            exc.CompileError,
+            "SQLite does not support on conflict "
+            "clause for column check constraint",
+            CreateTable(t).compile,
+            dialect=sqlite.dialect(),
+        )
+
+    def test_on_conflict_clause_primary_key_constraint(self):
+
+        meta = MetaData()
+        t = Table(
+            "n",
+            meta,
+            Column("id", Integer),
+            Column("x", String(30)),
+            PrimaryKeyConstraint("id", "x", sqlite_on_conflict="FAIL"),
+        )
+
+        self.assert_compile(
+            CreateTable(t),
+            "CREATE TABLE n ("
+            "id INTEGER NOT NULL, "
+            "x VARCHAR(30) NOT NULL, "
+            "PRIMARY KEY (id, x) ON CONFLICT FAIL)",
+            dialect=sqlite.dialect(),
+        )
+
 
 class InsertTest(fixtures.TestBase, AssertsExecutionResults):
 
