@@ -912,6 +912,14 @@ class BooleanEvalTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             dialect=self._dialect(False),
         )
 
+    def test_three_a_double(self):
+        c = column("x", Boolean)
+        self.assert_compile(
+            select([c]).where(~~c),
+            "SELECT x WHERE x = 1",
+            dialect=self._dialect(False),
+        )
+
     def test_three_b(self):
         c = column("x", Boolean)
         self.assert_compile(
@@ -928,6 +936,14 @@ class BooleanEvalTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             dialect=self._dialect(True),
         )
 
+    def test_four_double(self):
+        c = column("x", Boolean)
+        self.assert_compile(
+            select([c]).where(~~c),
+            "SELECT x WHERE x",
+            dialect=self._dialect(True),
+        )
+
     def test_five_a(self):
         c = column("x", Boolean)
         self.assert_compile(
@@ -2658,7 +2674,7 @@ class TupleTypingTest(fixtures.TestBase):
         eq_(expr.clauses[1].type._type_affinity, String)
         eq_(expr.clauses[2].type._type_affinity, LargeBinary()._type_affinity)
 
-    def test_type_coersion_on_eq(self):
+    def test_type_coercion_on_eq(self):
         a, b, c = (
             column("a", Integer),
             column("b", String),
@@ -2668,7 +2684,7 @@ class TupleTypingTest(fixtures.TestBase):
         expr = t1 == (3, "hi", "there")
         self._assert_types(expr.right)
 
-    def test_type_coersion_on_in(self):
+    def test_type_coercion_on_in(self):
         a, b, c = (
             column("a", Integer),
             column("b", String),
